#ifndef _EMMA_H_
#define _EMMA_H_
/*
 * linux/include/asm-mips/emma.h
 *
 * NEC EMMA registers.
 *
 */

//BHIF - host interface unit
#define EMMA_BHIF                   0xb0000000
#define EMMA_BHIF_VERSION           0xb0000000      // Version Register PD61130x chip version
#define EMMA_BHIF_STRAP_1           0xb0000010      // Strap 1 Register Indicates the system configuration mode CPU_BEND  (1<<0)  ROM_BEND  (1<<13)
#define EMMA_BHIF_STRAP_2           0xb0000014      // Strap 2 Register GCSSEL (1<<12)
#define EMMA_BHIF_STRAP_WRITE_1     0xb0000020      // Strap 1 Write Register Changes and Indicates the system configuration mode
#define EMMA_BHIF_STRAP_WRITE_2     0xb0000024      // Strap 2 Write Register
#define EMMA_BHIF_INT_ST_1          0xb0000030      // Interrupt Status 1 Register Indicates the general interrupt status of each unit
#define EMMA_BHIF_INT_ST_2          0xb0000034      // Interrupt Status 2 Register
#define EMMA_BHIF_INT_EN_1          0xb0000040      // Interrupt Enable 1 Register Enables/Masks the general interrupts
#define EMMA_BHIF_INT_EN_2          0xb0000044      // Interrupt Enable 2 Register
#define EMMA_BHIF_NMI_ST            0xb0000050      // NMI Status Register Indicates the priority interrupt (NMI for CPU) status
#define EMMA_BHIF_NMI_EN            0xb0000060      // NMI Enable Register Enables/Masks the priority interrupts (NMI for CPU)
#define EMMA_BHIF_BUS_PRIOR         0xb0000070      // Bus Priority Register Changes Priority between CPU and BEPU
#define EMMA_BHIF_BERR_CPUMSK       0xb00000c0      // Bus error CPU Mask Masks CPU access
#define EMMA_BHIF_BERR_CMTRG        0xb00000c4      // Bus error CPU Mask Trigger Updates CPU Mask Status
#define EMMA_BHIF_CLK_SEL_1         0xb0000100      // Clock Select 1 Register Selects Audio Clock, EMMA_MPEG Clock and IDE Clock
#define EMMA_BHIF_CLK_SEL_2         0xb0000104      // Clock Select 2 Register Selects Memory Clock
#define EMMA_BHIF_CLK_SEL_3         0xb0000108      // Clock Select 3 Register Selects TS Clock
#define EMMA_BHIF_CLK_MSK           0xb0000110      // Clock Mask Register Mask the Clocks that PLL Outputs
#define EMMA_BHIF_CLK_MCLKDIV       0xb0000120      // Clock MCLK Divider Register Selects MCLK Divider Value (M, N, P)
#define EMMA_BHIF_CLK_MCLKRST       0xb0000130      // Clock MCLK Reset Register Initialises MCLK Divider

//BARB -
#define EMMA_BARB                   0xb0000200

//BROI - rom interface
#define EMMA_BROI_FN_FSM            0xb0000400
#define EMMA_BROI_OAT_FAT           0xb0000404
#define EMMA_BROA_WERR_WM           0xb0000408

//OGIO - GIO INTERFACE
#define EMMA_OGIO                   0xb0000800
#define EMMA_OGIO_AC_CNT            0xb0000800
#define EMMA_OGIO_AC_CNT_GAT0_6_HCLKS   0x3
#define EMMA_OGIO_AC_CNT_RWMOD      (1 << 8)
#define EMMA_OGIO_AC_CNT_BENDIAN    (1 << 9)
#define EMMA_OGIO_CS0_BASE          0xb4000000
#define EMMA_OGIO_CS1_BASE          0xb4400000

//MPEG - MPEG Decoder
#define EMMA_MPEG                   0xb0002000
#define EMMA_MPEG_RESET             0xb0002000
#define EMMA_MPEG_BS_CTL_MAIN_CH    0xb0002010
#define EMMA_MPEG_BS_PTR_MAIN_CH    0xb0002014
#define EMMA_MPEG_START_MODE        0xb0002018
#define EMMA_MPEG_START_CTL         0xb000201c
#define EMMA_MPEG_START_STATUS      0xb0002020
#define EMMA_MPEG_START_STATUS2     0xb0002024

#define EMMA_MPEG_DEC_OV            0xb0002050
#define EMMA_MPEG_DEC_MODE          0xb0002054
#define EMMA_MPEG_DEC_CTL           0xb0002058
#define EMMA_MPEG_WAIT_COND         0xb000205c

// JPEG registers
#define EMMA_MPEG_JPEG_WADR         0xb0002060
#define EMMA_MPEG_JPEG_MODE         0xb0002064
#define EMMA_MPEG_JPEG_CTL          0xb0002068

#define EMMA_MPEG_TIMEOUT_MAIN      0xb0002070
#define EMMA_MPEG_TIMEOUT_SUB       0xb0002074
#define EMMA_MPEG_MAIN_STATUS       0xb0002080
#define EMMA_MPEG_ZERO23_STAT       0xb0002084
#define EMMA_MPEG_DEC_STATUS        0xb0002088
#define EMMA_MPEG_BRL_STATUS        0xb000208c
#define EMMA_MPEG_SUB_FIFO_ST       0xb0002090
#define EMMA_MPEG_TIMEOUT_STAT      0xb0002094
#define EMMA_MPEG_VLC_ERROR         0xb0002098
#define EMMA_MPEG_BRL_RD            0xb0002100
#define EMMA_MPEG_BRL_SHIFT         0xb0002104      // WO

#define EMMA_MPEG_MATRIX_I          0xb0002200
#define EMMA_MPEG_MATRIX_N          0xb0002240
#define EMMA_MPEG_PARAM_SEQ         0xb0002280      // This register holds the width and height of the actual frame
#define EMMA_MPEG_PARAM_PICH        0xb0002284
#define EMMA_MPEG_PARAM_PICE        0xb0002288
#define EMMA_MPEG_MC_MODE           0xb000228c
#define EMMA_MPEG_CPU_INT           0xb00022c0
#define EMMA_MPEG_CPU_MASK          0xb00022c4

// EMMA_MPEG interrupt flags
#define EMMA_MPEG_INT_START_MAIN    (1)
#define EMMA_MPEG_INT_START_SUB     (1 << 1)
#define EMMA_MPEG_INT_VLC_ERR       (1 << 2)
#define EMMA_MPEG_INT_MPEG_DONE     (1 << 3)
#define EMMA_MPEG_INT_JPEG_DONE     (1 << 4)
#define EMMA_MPEG_INT_TIME_MAIN     (1 << 5)
#define EMMA_MPEG_INT_TIME_SUB      (1 << 6)
#define EMMA_MPEG_INT_WAIT          (1 << 7)


// Video Display Processor
#define EMMA_DSCL                   0xb0004000      // scaler control
#define EMMA_DGRP                   0xb0004100      // graphics (OSD & cursor) control
#define EMMA_DALP                   0xb0004200      // alpha control
#define EMMA_DTCC                   0xb0004300      // display teletext and closed caption
#define EMMA_DSNC                   0xb0004400      // sync generator control
#define EMMA_DTOP                   0xb0004600      // timing, global control and interrupts

// Control for the 3 scalers (mpeg/static/video in)
#define EMMA_DSCL_SCL1_HDRTOP       0xb0004000      // scaler 1 top header addr
#define EMMA_DSCL_SCL1_HDRBOT       0xb0004004      // scaler 1 bottom header addr
#define EMMA_DSCL_SCL2_HDRTOP       0xb0004010      // scaler 2 top header addr
#define EMMA_DSCL_SCL2_HDRBOT       0xb0004014      // scaler 2 bottom header addr
#define EMMA_DSCL_SCL3_HDRTOP       0xb0004020      // scaler 3 top header addr
#define EMMA_DSCL_SCL3_HDRBOT       0xb0004024      // scaler 3 bottom header addr
#define EMMA_DSCL_CUR_DSPLINE       0xb0004030      // scaler current display line ctl

// Graphics processor control - OSD and cursor planes
#define EMMA_DGRP_RELCTR            0xb0004100      // Reload control
#define EMMA_DGRP_OSD1CTR           0xb0004104      // OSD #1 control
#define EMMA_DGRP_OSD2CTR           0xb0004108      // OSD #2 control
#define EMMA_DGRP_HWCCTR            0xb000410c      // hardware cursor control
#define EMMA_DGRP_HWCSTART          0xb0004110      // hardware cursor start position

// Alpha control
#define EMMA_DALP_ALPHA1            0xb0004200      // Alpha value 1
#define EMMA_DALP_ALPHA2            0xb0004204      // Alpha value 2
#define EMMA_DALP_ALPHA3            0xb0004208      // Alpha value 3
#define EMMA_DALP_ALPHA_CTR         0xb000420c      // Alpha blending control
#define EMMA_DALP_BCKGND1           0xb0004210      // background #1 color
#define EMMA_DALP_BORDER1           0xb0004214      // border #1 color)
#define EMMA_DALP_BCKGND2           0xb0004218      // background #2 color
#define EMMA_DALP_BORDER2           0xb000421c      // border #2 color)
#define EMMA_DALP_FBLANK            0xb0004220      // fast blanking signal control
#define EMMA_DALP_ALPHALMT0         0xb0004224      // Alpha output limit 0
#define EMMA_DALP_ALPHALMT1         0xb0004228      // Alpha output limit 1
#define EMMA_DALP_ALPHALMT2         0xb000422c      // Alpha output limit 2

// Display teletext and closed caption
#define EMMA_DTCC_TT_CC_CTR         0xb0004300      // teletext and closed caption ctl
#define EMMA_DTCC_CC_DATA_CTR       0xb0004304      // closed caption data control
#define EMMA_DTCC_TT_CC_FIELD       0xb0004308      // teletext field control

// Sync generator control
#define EMMA_DSNC_SYNC_CTLFLG       0xb0004400      // sync control flag
#define EMMA_DSNC_SYNC_RELOAD       0xb0004404      // sync reload timing

#define EMMA_DSNC_SYNC1_HTOT        0xb0004408      // sync #1 horizontal
#define EMMA_DSNC_SYNC1_HSYNC       0xb000440c      // sync #1 HSYNC parameter
#define EMMA_DSNC_SYNC1_VTOT        0xb0004410      // sync #1 vertical
#define EMMA_DSNC_SYNC1_VSYNC       0xb0004414      // sync #1 VSYNC paramter #1
#define EMMA_DSNC_SYNC1_V2SYNC      0xb0004418      // sync #1 VSYNC paramter #2
#define EMMA_DSNC_SYNC1_HVOFF       0xb000441c      // sync #1 HOR/VERT offset
#define EMMA_DSNC_SYNC1_CTOFF       0xb0004420      // sync #1 count offset
#define EMMA_DSNC_SYNC1_HCUT1       0xb0004424      // sync #1 horiz counter cut1
#define EMMA_DSNC_SYNC1_HCUT2       0xb0004428      // sync #1 horiz counter cut2

#define EMMA_DSNC_SYNC2_HTOT        0xb000442c      // sync #2 horizontal
#define EMMA_DSNC_SYNC2_HSYNC       0xb0004430      // sync #2 HSYNC parameter
#define EMMA_DSNC_SYNC2_VTOT        0xb0004434      // sync #2 vertical
#define EMMA_DSNC_SYNC2_VSYNC       0xb0004438      // sync #2 VSYNC paramter #1
#define EMMA_DSNC_SYNC2_V2SYNC      0xb000443c      // sync #2 VSYNC paramter #2
#define EMMA_DSNC_SYNC2_HVOFF       0xb0004440      // sync #2 HOR/VERT offset
#define EMMA_DSNC_SYNC2_CTOFF       0xb0004444      // sync #2 count offset
#define EMMA_DSNC_SYNC2_HCUT1       0xb0004448      // sync #2 horiz counter cut1
#define EMMA_DSNC_SYNC2_HCUT2       0xb000444c      // sync #2 horiz counter cut2

#define EMMA_DSNC_SYNC_EXTRST       0xb0004450      // sync 1&2 timing for extern mode

#define EMMA_DSNC_SYNC3_HTOT        0xb0004454      // sync #3 horizontal
#define EMMA_DSNC_SYNC3_HSYNC       0xb0004458      // sync #3 HSYNC parameter
#define EMMA_DSNC_SYNC3_VTOT        0xb000445c      // sync #3 vertical
#define EMMA_DSNC_SYNC3_VSYNC       0xb0004460      // sync #3 VSYNC paramter #1
#define EMMA_DSNC_SYNC3_V2SYNC      0xb0004464      // sync #3 VSYNC paramter #2

#define EMMA_DSNC_VALID_TVC         0xb0004468      // valid sig active top area
#define EMMA_DSNC_VALID_BVC         0xb000446c      // valid sig active bottom area
#define EMMA_DSNC_VALID_HC          0xb0004470      // valid sig active hcounter area

#define EMMA_DSNC_SYNC1_VC12        0xb0004474      // sync #1 interrupt timing vc 1,2
#define EMMA_DSNC_SYNC1_VC34        0xb0004478      // sync #1 interrupt timing vc 3,4
#define EMMA_DSNC_SYNC2_VC12        0xb000447c      // sync #2 interrupt timing vc 1,2
#define EMMA_DSNC_SYNC2_VC34        0xb0004480      // sync #2 interrupt timing vc 3,4
#define EMMA_DSNC_SYNC3_VC12        0xb0004484      // sync #3 interrupt timing vc 1,2
#define EMMA_DSNC_SYNC3_VC34        0xb0004488      // sync #3 interrupt timing vc 3,4

// timing, global control, and interrupts
#define EMMA_DTOP_DSCL1_RELD        0xb0004600      // DSCL1 reload timing
#define EMMA_DTOP_DSCL2_RELD        0xb0004604      // DSCL2 reload timing
#define EMMA_DTOP_DSCL3_RELD        0xb0004608      // DSCL3 reload timing
#define EMMA_DTOP_DOSD1_RELD        0xb000460c      // OSD 1 reload timing
#define EMMA_DTOP_DOSD2_RELD        0xb0004610      // OSD 2 reload timing
#define EMMA_DTOP_DHWC_RELD         0xb0004614      // hardware cursor reload timing
#define EMMA_DTOP_DALP_RELD         0xb000461c      // DALP reload timing

#define EMMA_DTOP_SYNC_SEL          0xb0004700      // sync1 or sync2 select control
#define EMMA_DTOP_SYNC_SELST        0xb0004704      // sync1 or sync2 select status
#define EMMA_DTOP_SYNC12_VFLD       0xb0004708      // sync1 and sync2 vcount fields
#define EMMA_DTOP_SYNC3_VFLD        0xb000470c      // sync3 vcount field value

#define EMMA_DTOP_INT1_SEL          0xb0004710      // interrupt 1
#define EMMA_DTOP_INT1_EN           0xb0004714      // interrupt 1 enable
#define EMMA_DTOP_STAT1_SEL         0xb0004718      // interrupt 1 status
#define EMMA_DTOP_INT2_SEL          0xb000471c      // interrupt 2
#define EMMA_DTOP_INT2_EN           0xb0004720      // interrupt 2 enable
#define EMMA_DTOP_STAT2_SEL         0xb0004724      // interrupt 2 status

#define EMMA_DTOP_SOFT_RST          0xb0004728      // software resets

#define EMMA_GBLT                   0xb0006000
#define EMMA_GDVI                   0xb0006800

//AUD - Audio Interface
#define EMMA_AUD                    0xb0007000      // MPEG AUDIO BASE *
#define EMMA_AUD_DDB0               0xb0007000      // command Interface     0x00
#define EMMA_AUD_ASM                0xb0007002      // memory select         0x08
#define EMMA_AUD_RAMADR             0xb0007006      // memory top address    0x18
#define EMMA_AUD_RAMDAT             0xb0007007      // memory data           0x1c
#define EMMA_AUD_DPUTSTA0           0xb0007008      // for patch Config      0x20
#define EMMA_AUD_DPUTSTA1           0xb0007009      // for patch Config      0x24
#define EMMA_AUD_DPUTSTA2           0xb000700a      // for patch Config      0x28
#define EMMA_AUD_DPUTSDA0           0xb000700b      // for patch Config      0x2C
#define EMMA_AUD_DPUTSDA1           0xb000700c      // for patch Config      0x30
#define EMMA_AUD_DPUTSDA2           0xb000700d      // for patch Config      0x34
#define EMMA_AUD_SPUTSTA0           0xb000700e      // for patch Config      0x38
#define EMMA_AUD_SPUTSTA1           0xb000700f      // for patch Config      0x3c
#define EMMA_AUD_SPUTSTA2           0xb0007010      // for patch Config      0x40
#define EMMA_AUD_SPUTSDA0           0xb0007011      // for patch Config      0x44
#define EMMA_AUD_SPUTSDA1           0xb0007012      // for patch Config      0x48
#define EMMA_AUD_SPUTSDA2           0xb0007013      // for patch Config      0x4C
#define EMMA_AUD_TSEN               0xb0007014      // Trap set enable       0x50
#define EMMA_AUD_RST                0xb0007040      // software reset        0x100
#define EMMA_AUD_INT                0xb0007044      // interrupts            0x110
#define EMMA_AUD_INTEN              0xb0007045      // interrupt enables     0x114
#define EMMA_AUD_INTSTA             0xb0007046      // interrupt status      0x118
#define EMMA_AUD_SICTRL             0xb0007048      // dma read ctrl, cha0,1 0x120
#define EMMA_AUD_ESIFORM            0xb000704c      // Ext input stream fmt  0x130
#define EMMA_AUD_ESICTRL            0xb000704d      // DMA ctrl              0x134
#define EMMA_AUD_SPDCTRL            0xb000705c      // spdif ctrl            0x170
#define EMMA_AUD_SPDCHSTA1          0xb000705d      // spdif mode chan stat  0x174
#define EMMA_AUD_SPDCHSTA2          0xb000705e      // pdif mode chan stat   0x178


//VENC - Video Encoder
#define EMMA_VENC                   0xb0008000
#define EMMA_VENC_VEE_CTR1          0xb0008010      // Control 1
#define EMMA_VENC_VEE_CTR2          0xb0008014      // Control 2
#define EMMA_VENC_VEE_CTR3          0xb0008018      // Control 3
#define EMMA_VENC_VEE_CTR4          0xb000801c      // Control 4
#define EMMA_VENC_VEE_CTR5          0xb0008020      // Control 5
#define EMMA_VENC_VEE_CTR6          0xb0008024      // Control 6
#define EMMA_VENC_VEE_CTR7          0xb0008028      // Control 7

#define EMMA_VENC_VEE_TTE1          0xb000802c      // Teletext Field 1
#define EMMA_VENC_VEE_TTE2          0xb0008030      // Teletext Field 2
#define EMMA_VENC_VEE_SECAM1        0xb0008034      // SECAM control 1
#define EMMA_VENC_VEE_SECAM2        0xb0008038      // SECAM control 2
#define EMMA_VENC_VEE_CTR8_DBG      0xb000803c      // Control 8 and Debug
#define EMMA_VENC_VEE_CTR9          0xb0008040      // Control 9
#define EMMA_VENC_VEE_CTR10         0xb0008044      // Control 10
#define EMMA_VENC_VEE_VID_WSS       0xb0008048      // Video ID and WSS control
#define EMMA_VENC_VEE_CC1           0xb000804c      // Closed Caption Field 1
#define EMMA_VENC_VEE_CC2           0xb0008050      // Closed Caption Field 2
#define EMMA_VENC_VEE_TTE3          0xb0008054      // Teletext Line Disable

//EMMA_VENC_VEE_VID_WSS register details:
// 0000 0000 0000 0000 O.AA AABB BBCC CDDD
// 0 ... always 0
// O ... WSS on/off
// . ... not used or unknown
// A ... Group 1 (see EN 300294 for details)
// B ... Group 2
// C ... Group 3
// D ... Group 4





//XDMA - DMA Controller
// The EMMA_MPEG decoder receives its slice data from the DMA controller
#define EMMA_XDMA                   0xb0009000
#define EMMA_XDMA_VBUF_RST          0xb0009004      // reset video buffers
#define EMMA_XDMA_ABUF_RST          0xb0009008      // reset audio buffers
#define EMMA_XDMA_VBUF_ENA          0xb0009014      // enable video buffer requests
#define EMMA_XDMA_REQ_ENA_ABUF      0xb0009018      // enable video buffer requests
#define EMMA_XDMA_REQ_MASK          0xb0009020      // DMA stop/restart flags
#define EMMA_XDMA_VBUF0_ADR         0xb0009060      // video buffer 0 start/end address
#define EMMA_XDMA_VBUF1_ADR         0xb0009064      // video buffer 1 start/end address
#define EMMA_XDMA_VBUFM_RPTR        0xb0009070      // video buffer/main read pointers
#define EMMA_XDMA_VBUFS_RPTR        0xb0009074      // video buffer/sub  read pointers
#define EMMA_XDMA_VBUF0_CUR         0xb0009080      // video buffer 0 current size x32
#define EMMA_XDMA_VBUF1_CUR         0xb0009090      // video buffer 1 current size x32
#define EMMA_XDMA_VBUF0_NF          0xb0009084      // video buffer 0 nearly full x32
#define EMMA_XDMA_VBUF1_NF          0xb0009094      // video buffer 1 nearly full x32
#define EMMA_XDMA_VBUF0_FULL        0xb0009088      // video buffer 0 full level x32
#define EMMA_XDMA_VBUF1_FULL        0xb0009098      // video buffer 1 full level x32
#define EMMA_XDMA_VBUF0_NE          0xb000908c      // video buffer 0 nearly empty x32
#define EMMA_XDMA_VBUF1_NE          0xb000909c      // video buffer 1 nearly empty x32
#define EMMA_XDMA_VBUF_ADD          0xb00090a0      // video buffer data add register
#define EMMA_XDMA_VBUF0_WPTR        0xb00090a4      // video buffer 0 write pointer
#define EMMA_XDMA_VBUF1_WPTR        0xb00090a8      // video buffer 1 write pointer
#define EMMA_XDMA_ABUF0_ADR         0xb00090b0      // Aud buf 0 address
#define EMMA_XDMA_ABUF1_ADR         0xb00090b4      // Aud buf 1 address
#define EMMA_XDMA_ABUF0_RPTR        0xb00090b8      // Aud buf 0 read ptr
#define EMMA_XDMA_ABUF1_RPTR        0xb00090bc      // Aud buf 1 read ptr
#define EMMA_XDMA_ABUF0_CUR_NUM     0xb00090c0      // Aud buf 0 cur # data units
#define EMMA_XDMA_ABUF0_NFUL_LVL    0xb00090c4      // Aud buf 0 nearly full level
#define EMMA_XDMA_ABUF0_FULL_LVL    0xb00090c8      // Aud buf 0 full level
#define EMMA_XDMA_ABUF0_NEMP_LVL    0xb00090cc      // Aud buf 0 nearly empty level
#define EMMA_XDMA_ABUF1_CUR_NUM     0xb00090d0      // Aud buf 1 cur # data units
#define EMMA_XDMA_ABUF1_NFUL_LVL    0xb00090d4      // Aud buf 1 nearly full level
#define EMMA_XDMA_ABUF1_FULL_LVL    0xb00090d8      // Aud buf 1 full level
#define EMMA_XDMA_ABUF1_NEMP_LVL    0xb00090dc      // Aud buf 1 nearly empty level 0xdc
#define EMMA_XDMA_TRANS_STAT        0xb00090f0      // Aud buf 1 nearly empty level 0xdc
#define EMMA_XDMA_ENA_STAT_VBUF     0xb00091c0      // video buffer enable status, RO
#define EMMA_XDMA_ENA_STAT_ABUF     0xb00091c4      // video buffer enable status, RO
#define EMMA_XDMA_BUF_OVERFLOW      0xb00091dc      // a&v buffer overflow flags, RO
#define EMMA_XDMA_INT_STATUS        0xb00091e0      // DMA controller interrupts status
#define EMMA_XDMA_INT_EN            0xb00091e4      // DMA controller interrupt enables

//TTSD -
#define EMMA_TTSD                   0xb000a000
#define EMMA_TTSD_PWM_DATA          0xb000a128
#define EMMA_TTSD_1ST_STC1          0xb000a250
#define EMMA_TTSD_1ST_STC2          0xb000a254
#define EMMA_TTSD_LATCHED_1ST_STC1  0xb000a258
#define EMMA_TTSD_LATCHED_1ST_STC2  0xb000a25c
#define EMMA_TTSD_1ST_PCR1          0xb000a248
#define EMMA_TTSD_1ST_PCR2          0xb000a24c
#define EMMA_TTSD_2ND_PCR1          0xb000a264
#define EMMA_TTSD_2ND_PCR2          0xb000a268
#define EMMA_TTSD_2ND_STC1          0xb000a26c
#define EMMA_TTSD_2ND_STC2          0xb000a270
#define EMMA_TTSD_x0                0xb000a3a0
#define EMMA_TTSD_x1                0xb000a604
#define EMMA_TTSD_PB_CURR_ADDRESS   0xb000a618




//WMIF - Memory Interface
// The EMMA_MPEG decoder renders its frames to areas set up in the mem controller
#define EMMA_WMIF                   0xb0010000
#define EMMA_WMIF_SOFT_RST          0xb0010000
#define EMMA_WMIF_CONFIG            0xb0010004
#define EMMA_WMIF_INIT              0xb0010008
#define EMMA_WMIF_REFCNT            0xb001000c
#define EMMA_WMIF_DISP_CFG          0xb0010010
#define EMMA_WMIF_MPEG_CFG          0xb0010014
#define EMMA_WMIF_DMA_CFG           0xb0010018
#define EMMA_WMIF_IDE_CFG           0xb001001c
#define EMMA_WMIF_TS_CFG            0xb0010020
#define EMMA_WMIF_DVI_CFG           0xb0010024
#define EMMA_WMIF_CPU_CFG           0xb0010028
#define EMMA_WMIF_BLT_CFG           0xb001002c
#define EMMA_WMIF_CNT_CFG           0xb0010030
#define EMMA_WMIF_REQCNT            0xb0010034
#define EMMA_WMIF_INT               0xb0010038
#define EMMA_WMIF_INT_EN            0xb001003c

// These registers hold pointers to the 8 EMMA_MPEG decode windows
#define EMMA_WMIF_ATBL0_Y           0xb0010040      // This register of the memory interface holds the address of Y-data of the first of 8 decoded MPEG frames
#define EMMA_WMIF_ATBL0_C           0xb0010044
#define EMMA_WMIF_ATBL1_Y           0xb0010048
#define EMMA_WMIF_ATBL1_C           0xb001004c
#define EMMA_WMIF_ATBL2_Y           0xb0010050
#define EMMA_WMIF_ATBL2_C           0xb0010054
#define EMMA_WMIF_ATBL3_Y           0xb0010058
#define EMMA_WMIF_ATBL3_C           0xb001005c
#define EMMA_WMIF_ATBL4_Y           0xb0010060
#define EMMA_WMIF_ATBL4_C           0xb0010064
#define EMMA_WMIF_ATBL5_Y           0xb0010068
#define EMMA_WMIF_ATBL5_C           0xb001006c
#define EMMA_WMIF_ATBL6_Y           0xb0010070
#define EMMA_WMIF_ATBL6_C           0xb0010074
#define EMMA_WMIF_ATBL7_Y           0xb0010078
#define EMMA_WMIF_ATBL7_C           0xb001007c

// The rest are still unexplained by NEC
#define EMMA_WMIF_YTBL_DISP_T0      0xb0010080
#define EMMA_WMIF_YTBL_DISP_T1      0xb0010084
#define EMMA_WMIF_YTBL_DISP_T2      0xb0010088
#define EMMA_WMIF_YTBL_DISP_T3      0xb001008c
#define EMMA_WMIF_YTBL_DISP_T4      0xb0010090

#define EMMA_WMIF_YTBL_DISP_B0      0xb00100a0
#define EMMA_WMIF_YTBL_DISP_B1      0xb00100a4
#define EMMA_WMIF_YTBL_DISP_B2      0xb00100a8
#define EMMA_WMIF_YTBL_DISP_B3      0xb00100ac
#define EMMA_WMIF_YTBL_DISP_B4      0xb00100b0

#define EMMA_WMIF_YTBL_MPEG_T0      0xb00100c0
#define EMMA_WMIF_YTBL_MPEG_T1      0xb00100c4
#define EMMA_WMIF_YTBL_MPEG_T2      0xb00100c8
#define EMMA_WMIF_YTBL_MPEG_T3      0xb00100cc
#define EMMA_WMIF_YTBL_MPEG_T4      0xb00100d0

#define EMMA_WMIF_YTBL_MPEG_B0      0xb00100e0
#define EMMA_WMIF_YTBL_MPEG_B1      0xb00100e4
#define EMMA_WMIF_YTBL_MPEG_B2      0xb00100e8
#define EMMA_WMIF_YTBL_MPEG_B3      0xb00100ec
#define EMMA_WMIF_YTBL_MPEG_B4      0xb00100f0

#define EMMA_WMIF_DUMMY             0xb00100f8
#define EMMA_WMIF_CLOCK_DELAY       0xb00100fc




//IDE
#define EMMA_ATA                    0xb0012003
#define EMMA_CU                     0xb0012900
#define EMMA_IDE                    0xb0012d20
#define EMMA_IDE_x0                 0xb0012d23      // pData        data register
#define EMMA_IDE_x1                 0xb0012d23      // pError       feature/error register
#define EMMA_IDE_x2                 0xb0012d27      // pFeature     feature/error register
#define EMMA_IDE_x3                 0xb0012d2b      // pSecCount    sector count register
#define EMMA_IDE_x4                 0xb0012d2f      // pLBALow      sector number register = LBA low register
#define EMMA_IDE_x5                 0xb0012d33      // pLBAMid      cylinder low register  = LBA mid register
#define EMMA_IDE_x6                 0xb0012d37      // pLBAHigh     cylinder high register = LBA high register
#define EMMA_IDE_x7                 0xb0012d3b      // pDevice      device/head register
#define EMMA_IDE_x8                 0xb0012d3f      // pStatus      command/status register
#define EMMA_IDE_x9                 0xb0012d3f      // pCommand     command/status register
#define EMMA_IDE_xa                 0xb0012d5b      // pAltStatus   alternate status register
#define EMMA_IDE_xb                 0xb0012d5b      // pDevControl  device control register




//PCI - PCI INTERFACE
#define EMMA_PCI_CBI                0xb0013000
#define EMMA_PCI_RST                0xb0013000
#define EMMA_PCI_ARBEN              0xb0013030
#define EMMA_PCI_TENDIAN            0xb0013070
#define EMMA_PCI_TENDIAN_ENDIAN (1 << 0)
#define EMMA_PCI_DMACTRL            0xb0013080
#define EMMA_PCI_ERR                0xb00130b8
#define EMMA_PCI_CTRL_L             0xb00130e0
#define EMMA_PCI_CTRL_H             0xb00130e4
#define EMMA_PCI_INIT0              0xb00130f0
#define EMMA_PCI_INIT1              0xb00130f8

#define EMMA_PCI_INIT_ADDR_MASK     0xffe00000
#define EMMA_PCI_INIT_CMD_IO        0x00000002
#define EMMA_PCI_INIT_CMD_MEM       0x00000006
#define EMMA_PCI_INIT_CMD_CFG       0x0000000a
#define EMMA_PCI_INIT_ACCESS_32     (1 << 4)
#define EMMA_PCI_INIT_CFG_TYPE1     (1 << 9)

#define EMMA_PCI_CMD_STS            0xb0013204
#define EMMA_PCI_BARC               0xb0013210
#define EMMA_PCI_BAR0               0xb0013240
#define EMMA_PCI_PCI                0xb0200000

#define EMMA_PCI_WIN0_BASE          0xb3000000
#define EMMA_PCI_WIN0_SIZE          0x01000000

#define EMMA_PCI_WIN1_BASE          0xb5000000
#define EMMA_PCI_WIN1_SIZE          0x08000000


// PCIMAP - Not part of the EMMA spec but closely tied to it.
#define EMMA_PCIMAP_SDRAM_BASE      0x00000000
#define EMMA_PCIMAP_IO_BASE         0x00000000
#define EMMA_PCIMAP_IO_SIZE         EMMA_PCI_WIN0_SIZE
#define EMMA_PCIMAP_MEM_BASE        0x30000000
#define EMMA_PCIMAP_MEM_SIZE        EMMA_PCI_WIN1_SIZE


//PBRD -
#define EMMA_PBRD                   0xb2000000

//PFUR - uart
#define EMMA_PFUR0                  0xb2001000
#define EMMA_PFUR1                  0xb2002000
#define EMMA_PFUR1_DATA             0xb2002000      // data /divisor
#define EMMA_PFUR1_IERD             0xb2002010      // int enable /divisor
#define EMMA_PFUR1_DIVL             0xb2002000      // divisor low bits
#define EMMA_PFUR1_DIVH             0xb2002010      // divisor high bits
#define EMMA_PFUR1_IIR              0xb2002020      // int ident register
#define EMMA_PFUR1_LCR              0xb2002030      // line control reg
#define EMMA_PFUR1_MCR              0xb2002040      // modem control reg
#define EMMA_PFUR1_LSR              0xb2002050      // line status reg
#define EMMA_PFUR1_MSR              0xb2002060      // modem status reg
#define EMMA_PFUR1_SCR              0xb2002070      // scratch register

//PURT - uart
#define EMMA_PURT0                  0xb2003000
#define EMMA_PURT1                  0xb2004000
#define EMMA_PURT0_ASIM             0xb2003000      // async serial intf mode reg
#define EMMA_PURT0_ERR              0xb2003020      // error reg
#define EMMA_PURT0_BRGC             0xb2003030      // baud rate gen ctrl reg
#define EMMA_PURT0_BRGMC            0xb2003040      // baud rate gen mode reg
#define EMMA_PURT0_DATA             0xb2003050      // rx and tx data reg
#define EMMA_PURT0_INT              0xb2003060      // interrupt register
#define EMMA_PURT0_INTM             0xb2003070      // interrupt mask register

//PSCI - serial
#define EMMA_PSCI0                  0xb2005000
#define EMMA_PSCI1                  0xb2006000
//PIIC - i2c
#define EMMA_PIIC0                  0xb2007000
#define EMMA_PIIC1                  0xb2008000
//PCIF -
#define EMMA_PCIF                   0xb2009000
//PCSI -
#define EMMA_PCSI                   0xb200a000
//ZTMR -
#define EMMA_ZTMR0                  0xb2010200
#define EMMA_ZTMR1                  0xb2010240
//ZRTC -
#define EMMA_ZRTC                   0xb2010400
//ZECT -
#define EMMA_ZECT                   0xb2010600

//ZCCT - Capture Timers
#define EMMA_ZCCT0                  0xb2010800
#define EMMA_ZCCT1                  0xb2010840
#define EMMA_ZCCT2                  0xb2010880
#define EMMA_ZCCT3                  0xb20108c0

#define EMMA_ZCCT0_TIMER_CMD        0xb2010800      // Timer Command Reg
#define EMMA_ZCCT0_TIMER_MODE       0xb2010804      // Timer Mode Reg
#define EMMA_ZCCT0_TIMER_VALUE      0xb2010808      // Timer Value Reg
#define EMMA_ZCCT0_TIMER_STATUS     0xb201080c      // Timer Status Reg
#define EMMA_ZCCT0_INT_STATUS       0xb2010810      // Timer Int Status Reg
#define EMMA_ZCCT0_EVENT_ACT        0xb2010814      // Timer event action Reg
#define EMMA_ZCCT0_CAPCOM_0         0xb2010818      // Timer Cap/Comp Val0 Reg
#define EMMA_ZCCT0_CAPCOM_1         0xb201081c      // Timer Cap/Comp Val1 Reg
#define EMMA_ZCCT0_CAPCOM_2         0xb2010820      // Timer Cap/Comp Val2 Reg
#define EMMA_ZCCT0_CAPCOM_3         0xb2010824      // Timer Cap/Comp Val3 Reg

//ZWDG - watchdog
#define EMMA_ZWDG                   0xb2010a00

//ZPIO - Programmable Pin Interface
#define EMMA_ZPIO                   0xb2010c00
#define EMMA_ZPIO_PORT_DATA_0       0xb2010c00
#define EMMA_ZPIO_PORT_DATA_1       0xb2010c04
#define EMMA_ZPIO_PORT_DIRECTION_0  0xb2010c08
#define EMMA_ZPIO_PORT_DIRECTION_1  0xb2010c0c
#define EMMA_ZPIO_INT_STATUS_0      0xb2010c10
#define EMMA_ZPIO_INT_STATUS_1      0xb2010c14
#define EMMA_ZPIO_INT_MODE_0        0xb2010c18
#define EMMA_ZPIO_INT_MODE_1        0xb2010c1c
#define EMMA_ZPIO_INT_CONDITION_A0  0xb2010c20
#define EMMA_ZPIO_INT_CONDITION_A1  0xb2010c24
#define EMMA_ZPIO_INT_CONDITION_B0  0xb2010c28
#define EMMA_ZPIO_INT_CONDITION_B1  0xb2010c2c
#define EMMA_ZPIO_PORT_WRITE_MASK_0 0xb2010c30
#define EMMA_ZPIO_PORT_WRITE_MASK_1 0xb2010c34

#define EMMA_TDSC                   0xb2010c30
#define EMMA_TSCA                   0xb2010e30
#define EMMA_TCAM                   0xb20111c4







// Scaler and OSD headers

// Resizing parameters for vertical/horizontal luma & chroma
typedef struct {
    dword       init;                               // resizing initial value
    dword       delta1;                             // resizing DDA delta1
    dword       limit;                              // resizing max delta
    dword       delta2;                             // resizing DDA delta2
    dword       delta3;                             // resizing DDA delta3
} EMMA_SCL_RESIZE_HEADER;

typedef struct {
    dword       control;                            // scaler control
    dword       size;                               // scaler data size
    dword       y_even;                             // scaler Y-even start address
    dword       y_odd;                              // scaler Y-odd start address
    dword       y_chr_even;                         // scaler chroma-even start address
    dword       y_chr_odd;                          // scaler chroma-odd start address
    dword       src_start;                          // scaler source start X & Y
    dword       src_size;                           // scaler source X & Y size
    dword       dsp_start;                          // scaler display start X & Y
    dword       dsp_size;                           // scaler display X & Y size
    dword       clip_start;                         // scaler clip start X & Y
    dword       clip_size;                          // scaler clip X & Y size
    dword       empty1;
    dword       empty2;
    dword       flt_control;                        // scaler filter control
    dword       flt_horiz;                          // scaler filter horizontal parameter
    dword       flt_vertlum;                        // scaler filter vertical luma parameter
    dword       flt_vertchr;                        // scaler filter vertial chroma parameter
    EMMA_SCL_RESIZE_HEADER  hy;                     // horizontal resizing luma parameters
    EMMA_SCL_RESIZE_HEADER  hc;                     // horizontal resizing chroma parameters
    EMMA_SCL_RESIZE_HEADER  vy;                     // vertical resizing luma parameters
    EMMA_SCL_RESIZE_HEADER  vc;                     // vertical resizing chroma parameters
    dword       chkey_max;                          // chroma-key maximum and control
    dword       chkey_min;                          // chroma-key minimum setting
    dword       alpha;                              // scaler alpha blending control
    dword       linebuf;                            // scaler line buffer control
    dword       window;                             // scaler window header link control
} EMMA_SCL_HEADER;

typedef struct {
    dword       alpha;                              // OSD alpha control
    dword       disp_start;                         // OSD display start X & Y
    dword       disp_size;                          // OSD display X & Y size
    dword       tile_size;                          // OSD tile cell size
    dword       clut_addr;                          // OSD color look-up table address
    dword       alph_addr;                          // OSD alpha lookup table address
    dword       data_ctl1;                          // OSD data control 1
    dword       data_ctl2;                          // OSD data control 2
    dword       chain_ctl;                          // OSD address chain control
    dword       alpha_sel1;                         // OSD alpha select #1
    dword       alpha_sel2;                         // OSD alpha select #2
    dword       alpha_sel3;                         // OSD alpha select #3
    dword       alpha_sel4;                         // OSD alpha select #4
    dword       alpha_sel5;                         // OSD alpha select #5
    dword       alpha_sel6;                         // OSD alpha select #6
    dword       alpha_sel7;                         // OSD alpha select #7
    dword       alpha_sel8;                         // OSD alpha select #8
    dword       not_used;
} EMMA_OSD_HEADER;

// Short OSD header when alpha select words not used
typedef struct {
    dword       alpha;                              // OSD alpha control
    dword       disp_start;                         // OSD display start X & Y
    dword       disp_size;                          // OSD display X & Y size
    dword       tile_size;                          // OSD tile cell size
    dword       clut_addr;                          // OSD color look-up table address
    dword       alph_addr;                          // OSD alpha lookup table address
    dword       data_ctl1;                          // OSD data control 1
    dword       data_ctl2;                          // OSD data control 2
    dword       chain_ctl;                          // OSD address chain control
    dword       not_used;
} EMMA_OSD_HEADER_S;

/* Hardware cursor header is 20 words, 80 bytes */

typedef struct {
    dword       curs_size;                          // HWC cursor size and timing
    dword       chain_ctl;                          // address of next HWC header
    dword       data_ctl1;                          // even line pointer & flags
    dword       data_ctl2;                          // odd line pointer
    dword       clut_tab[16];                       // cursor color lookup table data
} EMMA_HWC_HEADER;

#endif
