#ifndef _exTAP_
#define _exTAP_

#include "topfield.h"           // firmware internals

//#define exTAPdebug

//----------------------------------------------------------------------------
//
//error codes
//
enum {
    EXTAPERR_6 = -6,
    EXTAPERR_OTHER              = -5,
    EXTAPERR_BAD_MATCH          = -4,
    EXTAPERR_NOT_PATCHED        = -3,
    EXTAPERR_ALREADY_PATCHED    = -2,
    EXTAPERR_BAD_FIRMWARE       = -1,
    EXTAPERR_NONE               =  0,
};

//----------------------------------------------------------------------------
//
//patch stuff
//
typedef struct {
    word  sysid;
    dword revision;
    dword address;
}type_firmware;

typedef struct  {
    word  sysid;
    dword revision;
    int count;
    struct{
        dword addr;
        byte  from[4];
        byte  to[4];
    } info[0x20];
}type_patch;

typedef struct  {
    word  sysid;
    dword revision;
    dword addr;
    int   length;
    byte  from[0x100];
    byte  to[0x100];
}type_patch2;

//----------------------------------------------------------------------------
//
//hook stuff
//
#define EVT_HOOK    0xf100
#define EVT_MESSAGE 0xf200
#define MAX_HOOKS   16

typedef struct  {
    word  sysid;
    dword revision;
    dword enable;               //0=disabled, 1=enabled
    dword addr;                 //firmware hook address
    int   length;               //length
    byte  from[0x100];          //firmware original data
}type_hook;

typedef struct  {
    dword zero;
    dword at;
    dword v0;
    dword v1;
    dword a0;
    dword a1;
    dword a2;
    dword a3;

    dword t0;
    dword t1;
    dword t2;
    dword t3;
    dword t4;
    dword t5;
    dword t6;
    dword t7;

    dword s0;
    dword s1;
    dword s2;
    dword s3;
    dword s4;
    dword s5;
    dword s6;
    dword s7;

    dword t8;
    dword t9;
    dword k0;
    dword k1;
    dword gp;
    dword sp;
    dword s8;
    dword ra;
}type_hook_registers;

//----------------------------------------------------------------------------
//
//event stuff
//
//extended to give 0x100 bytes for the name, description
typedef struct
{
    word    evtId;                  //0x00
    dword   startTime;              //0x04
    dword   endTime;                //0x08
    word    duration;               //0x0c
    byte    runningStatus;          //0x0e
    char    eventName[256];         //0x0f
    char    description[256];       //0x8f,10f
    byte    parentalRating;         //0x10f,20f

    byte    satIdx;                 //0x110,210
    word    svcId;                  //0x112,212
    word    orgNetId;               //0x114,214
    word    tsId;                   //0x116,216
} TYPE_exTapEvent;

typedef struct
{
    byte    playMode;               //0x00    // Playback Mode, refer TYPE_PlayMode
    byte    trickMode;              //0x01    // Trick Mode, refer TYPE_TrickMode
    byte    speed;                  //0x02    // playback speed
    byte    svcType;                //0x03    // 0 = TV, 1 = Radio
    word    svcNum;                 //0x04
    word    duration;               //0x06
    dword   currentBlock;           //0x08
    dword   totalBlock;             //0x0c
    TYPE_File *file;                //0x10
    TYPE_exTapEvent evtInfo;        //0x14

    byte    repeatStatus;           //0x14+0x118
    byte    reserved[3];            //0x14+0x118+3
} TYPE_exPlayInfo;

//----------------------------------------------------------------------------
//some functions
extern int   exTAP_Init(void);
//----------------------------------------------------------------------------
extern dword exTAP_GetFirmwareAddress(type_firmware *firmware);
extern dword exTAP_CallFirmware(void *pFunction, dword p0, dword p1, dword p2, dword p3, dword p4, dword p5, dword p6, dword p7, dword p8, dword p9, dword p10);
extern dword exTAP_CallTap(int ident, void *pFunction, dword p0, dword p1, dword p2, dword p3, dword p4, dword p5, dword p6, dword p7, dword p8, dword p9, dword p10);
extern int   exTAP_SendMessage(dword param1, dword param2);
extern int   exTAP_ApplyPatch(type_patch *patch);
extern int   exTAP_RemovePatch(type_patch *patch);
extern int   exTAP_isPatched(type_patch *patch, int *index);
extern int   exTAP_ApplyHook(type_hook *hook);
extern int   exTAP_RemoveHook(int id);
//----------------------------------------------------------------------------
extern TYPE_exTapEvent *exTAP_GetEvent(int chType, int chNum, int *eventNum);
extern TYPE_exTapEvent *exTAP_GetCurrentEvent(int chType, int chNum);
extern byte *exTAP_EPG_GetExtInfo(TYPE_exTapEvent *tapEvtInfo);
extern bool  exTAP_EPG_DeleteEvent(int chType, int chNum, dword evtid);
extern bool  exTAP_EPG_UpdateEvent(int chType, int chNum, dword evtid, dword runningStatus);
extern int   exTAP_EPG_DeleteAllEvents(void);
extern int   exTAP_EPG_GetNumEvents(void);
extern int   exTAP_GetCurrentGenre(int chType, int chNum, int *nibble1, int *nibble2);
extern int   exTAP_GetGenre(TYPE_TapEvent *tapEvtInfo, int *nibble1, int *nibble2);
extern int   exTAP_EPG_Resize(dword short_size, dword ext_size);
extern int   exTAP_EPG_GetSize(int type);
extern int   exTAP_EIT_Schedule(int enable);
extern int   exTAP_EIT_NowNext(int enable);
extern int   exTAP_EPG_Stick(int enable);
//----------------------------------------------------------------------------
extern int   exTAP_Key_Extend(int enable);
extern int   exTAP_Key_Extend2(int enable);
extern int   exTAP_MHEG_Disable(int disable);      //0=mheg on, 1=mheg off
extern int   exTAP_MHEG_isPatched();               //0=no, 1=yes
extern dword exTAP_MHEG_Status(void);              //returns mheg register group, 0=off, 3=idle, 5=semi-active, 4=active
extern int   exTAP_TAP_Load(char *filename);
extern int   exTAP_Shutdown(void);
//----------------------------------------------------------------------------
extern int   exTAP_LED_GetStatus(void);
extern int   exTAP_LED_SetControl(int enable);
extern int   exTAP_LED_GetControl(void);
extern void  exTAP_LED_SendData(byte mask, byte *pData);
extern int   exTAP_LED_Translate(const char *pIn, byte *pOut);
extern int   exTAP_LED_GetMap(char ascii);
extern int   exTAP_LED_SetMap(char ascii, byte sevenseg);
//----------------------------------------------------------------------------
extern int   exTAP_GetUTCTime(word *mjd, byte *hour, byte *minute);
extern dword exTAP_UTCtoLocalTime(dword utc);
//----------------------------------------------------------------------------

#endif //_exTAP_
